/* Copyright (c) 2005, NIF File Format Library and Tools
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.

   * Redistributions in binary form must reproduce the above
     copyright notice, this list of conditions and the following
     disclaimer in the documentation and/or other materials provided
     with the distribution.

   * Neither the name of the NIF File Format Library and Tools
     project nor the names of its contributors may be used to endorse
     or promote products derived from this software without specific
     prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE. */

#ifndef _NIFLIB_H_
#define _NIFLIB_H_

//--Includes--//
#include <iostream>
#include <fstream>
#include <string>
#include <map>
#include <exception>
#include <stdexcept>
#include <algorithm>
#include <functional>
#include <numeric>
#include <vector>
#include <list>
#include <map>
using namespace std;

//--Forward Declarations of Classes & Structs--//
class IAttr;
class IBlock;
class IShapeData;
class ITriShapeData;
class ISkinData;
class IKeyframeData;
class ITextKeyExtraData;
class IMorphData;
class ITriStripsData;
class IColorData;
class IFloatData;
class IPosData;
class INode;
class blk_ref;
class attr_ref;
struct blk_link;
struct Texture;
struct TextureSource;
struct BoundingBox;
struct ConditionalInt;
struct SkinWeight;

//--Constants--//

//Interface IDs
const int ID_TRI_SHAPE_DATA = 0; /*!< ID for ITriShapeData Interface */  
const int ID_SKIN_DATA = 1; /*!< ID for ISkinData Interface */ 
const int ID_NODE = 2; /*!< ID for INode Interface */ 
const int ID_KEYFRAME_DATA = 3; /*!< ID for IKeyframeData Interface */ 
const int ID_TEXT_KEY_EXTRA_DATA = 4; /*!< ID for ITextKeyExtraData Interface */ 
const int ID_MORPH_DATA = 5; /*!< ID for IMorphData Interface */ 
const int ID_SHAPE_DATA = 6; /*!< ID for IShapeData Interface */ 
const int ID_TRI_STRIPS_DATA = 7; /*!< ID for ITriStripsData Interface */ 
const int ID_COLOR_DATA = 8; /*!< ID for IColorData Interface */ 
const int ID_FLOAT_DATA = 9; /*!< ID for IFloatData Interface */ 
const int ID_POS_DATA = 10; /*!< ID for IPosData Interface */ 


/*!
 * This enum contains all the attribute types used by Niflib.
 */
enum AttrType {
	attr_int, /*!< Integer Attribute.  Holds a 32-bit integer. */  
	attr_short, /*!< Short Integer Attribute.  Holds a 16-bit integer. */ 
	attr_byte, /*!< Byte Attribute.  Holds an 8-bit integer. */ 
	attr_float, /*!< Float Attribute.  Holds a 32-bit floating point number. */ 
	attr_float3, /*!< Float3 Attribute.  Holds a Float3 structure. */ 
	attr_string, /*!< String Attribute.  Holds a text string. */ 
	attr_link, /*!< Link Attribute.  Links to one other Nif block lower in the Nif tree. */ 
	attr_flags, /*!< Flags Attribute.  Holds a set of 16 bit flags whos function depends on the block that uses them. */ 
	attr_matrix33, /*!< Matrix33 Attribute.  Holds a Matrix33 structure. */ 
	attr_linkgroup, /*!< Link Group Attribute.  Links to several other Nif blocks lower in the Nif tree. */ 
	attr_bones, /*!< Bones Attribute.  Automatic. */ 
	attr_bbox, /*!< Bounding Box Attribute.  Holds a BoundingBox structure. */ 
	attr_condint, /*!< Conditional Integer Attribute.  Holds a ConditionalInt structure. */ 
	attr_vertmode, /*!< Vertex Mode Attribute.  Holds an integer that corresponds to the vertex mode. */ 
	attr_lightmode, /*!< Light Mode Attribute.  Holds an integer that corresponds to the light mode. */ 
	attr_texture, /*!< Texture Attribute.  Holds a Texture structure but ignores the bump map information. */ 
	attr_bumpmap, /*!< Bump Map Atrribute  Holds a Texture structure and uses the bump map information. */ 
	attr_applymode, /*!< Apply Mode Attribute.  Holds an integer that corresponds to the apply mode. */ 
	attr_texsource, /*!< Texture Source Attribute.  Holds a Texture Source structure. */ 
	attr_pixellayout, /*!< Light Mode Attribute.  Holds an integer that corresponds to the light mode. */ 
	attr_mipmapformat, /*!< Mipmap Format Attribute.  Holds an integer that corresponds to the mipmap format. */ 
	attr_alphaformat, /*!< Alpha Format Attribute.  Holds an integer that corresponds to the alpha format. */ 
	attr_controllertarget, /*!< Controller Target Attribute.  Automatic. */ 
	attr_skeletonroot, /*!< Skeleton Root Attribute.  Automatic. */ 
	attr_particlegroup, /*!< Particle Group Attribute. */ 
	attr_lodrangegroup, /*!< Level of Detail Range Group Attribute. */ 
	attr_vector3, /*!< Vector3 Attribute.  Holds a Float3 structure that corresponds to a vector in 3D space. */ 
	attr_color3, /*!< Color3 Attribute.  Holds a Float3 structure that corresponds to an RGB color. */ 
	attr_parent /*!< Parent Attribute.  Automatic. */ 
};

//NIF Versions
const unsigned int VER_4_0_0_2 = 0x04000002; /*!< Nif Version 4.0.0.2 */ 
const unsigned int VER_4_1_0_12 = 0x0401000C; /*!< Nif Version 4.1.0.12 */ 
const unsigned int VER_4_2_0_2 = 0x04020002; /*!< Nif Version 4.2.0.2 */ 
const unsigned int VER_4_2_1_0 = 0x04020100; /*!< Nif Version 4.2.1.0 */ 
const unsigned int VER_4_2_2_0 = 0x04020200; /*!< Nif Version 4.2.2.0 */ 
const unsigned int VER_10_0_1_0 = 0x0A000100; /*!< Nif Version 10.0.1.0 */ 
const unsigned int VER_10_1_0_0 = 0x0A010000; /*!< Nif Version 10.1.0.0 */ 
const unsigned int VER_10_2_0_0 = 0x0A020000; /*!< Nif Version 10.2.0.0 */ 
const unsigned int VER_20_0_0_4 = 0x14000004; /*!< Nif Version 20.0.0.4 */ 

/*! Specifies the availiable texture clamp modes.  That is, the behavior of pixels outside the range of the texture.*/
enum TexClampMode {
	CLAMP_S_CLAMP_T = 0, /*!< Clamp in both directions. */ 
	CLAMP_S_WRAP_T = 1, /*!< Clamp in the S direction but wrap in the T direction. */ 
	WRAP_S_CLAMP_T = 2, /*!< Wrap in the S direction but clamp in the T direction. */ 
	WRAP_S_WRAP_T = 3 /*!< Wrap in both directions. */ 
};

/*! Specifies the availiable texture filter modes.  That is, the way pixels within a texture are blended together when textures are displayed on the screen at a size other than their original dimentions.*/
enum TexFilterMode {
	FILTER_NEAREST = 0, /*!< Simply uses the nearest pixel.  Very grainy. */ 
	FILTER_BILERP = 1, /*!< Uses bilinear filtering. */ 
	FILTER_TRILERP = 2, /*!< Uses trilinear filtering. */ 
	FILTER_NEAREST_MIPNEAREST = 3, /*!< Uses the nearest pixel from the mipmap that is closest to the display size. */ 
	FILTER_NEAREST_MIPLERP = 4, /*!< Blends the two mipmaps closest to the display size linearly, and then uses the nearest pixel from the result. */ 
	FILTER_BILERP_MIPNEAREST = 5, /*!< Uses the closest mipmap to the display size and then uses bilinear filtering on the pixels. */ 
};

#ifndef NULL
#define NULL 0
#endif


/*!
 *   This enum contains all the animation key types used by Niflib.
 */
enum KeyType {
	LINEAR_KEY = 1, /*!< Use linear interpolation. */ 
	QUADRATIC_KEY = 2, /*!< Use quadratic interpolation.  Forward and back tangents will be stored.*/ 
	TBC_KEY = 3, /*!< Use Tension Bias Continuity interpolation.  Tension, bias, and continuity will be stored.*/ 
	XYZ_ROTATION_KEY = 4 /*!< For use only with rotation data.  Separate X, Y, and Z keys will be stored instead of using quaternions. */ 
};

//--Main Functions--//

/*!
 * Reads the given file by file name and returns a vector of block references
 * \param file_name The name of the file to load, or the complete path if it is not in the working directory.
 * \return A vector of block references that point to all the blocks read from the Nif file.
 * 
 * <b>Example:</b> 
 * \code
 * blk_ref my_block = ReadNifList("test_in.nif");
 * \endcode
 * 
 * <b>In Python:</b>
 * \code
 * my_block = ReadNifList("test_in.nif")
 * \endcode
 * 
 * \sa ReadNifTree, WriteNifTree
 */
vector<blk_ref> ReadNifList( string const & file_name );

/*!
 * Reads the given file by file name and returns a reference to the root block.
 * \param file_name The name of the file to load, or the complete path if it is not in the working directory.
 * \return A block reference that points to the root of tree of data blocks contained in the NIF file.
 * 
 * <b>Example:</b> 
 * \code
 * blk_ref my_block = ReadNifTree("test_in.nif");
 * \endcode
 * 
 * <b>In Python:</b>
 * \code
 * my_block = ReadNifTree("test_in.nif")
 * \endcode
 * 
 * \sa ReadNifList, WriteNifTree
 */
blk_ref ReadNifTree( string const & file_name );

/*!
 * Creates a new NIF file of the given file name by crawling through the data tree starting with the root block given.  Automatically writes a kf file and an x/nif file if animation is present.
 * \param file_name The desired file name for the new NIF file.  The path is relative to the working directory unless a full path is specified.
 * \param root_block The root block to start from when writing out the NIF file.  All decedents of this block will be written to the file in tree-descending order.
 * \param version The version of the NIF format to use when writing a file.  Default is version 4.0.0.2.
 * 
 * <b>Example:</b> 
 * \code
 * blk_ref my_block = ReadNifTree("test_in.nif");
 * WriteNifTree( "test_out.nif", my_block );
 * \endcode
 * 
 * <b>In Python:</b>
 * \code
 * my_block = ReadNifTree("test_in.nif")
 * WriteNifTree( "test_out.nif", my_block )
 * \endcode
 * 
 * \sa ReadNifList, WriteNifTree
 */
void WriteNifTree( string const & file_name, blk_ref const & root_block, unsigned int version = VER_4_0_0_2 );

//// Returns list of all blocks in the tree rooted by root block.
//list<blk_ref> GetNifTree( blk_ref const & root_block );

////Returns the NIF spec version of a file, given a file name.
//string GetFileVersion(string file_name);


/*!
 * Sets whether data/hex areas are shown by asString functions
 * \param val A boolean value that specifies whether verbose mode should be turned on.  True = Verbose, False = Brief.
 * 
 * <b>Example:</b> 
 * \code
 * SetVerboseMode(true);
 * \endcode
 * 
 * <b>In Python:</b>
 * \code
 * SetVerboseMode(True)
 * \endcode
 * 
 * \sa IBlock::asString, IAttr::asString
 */
void SetVerboseMode( bool val );

/*!
 * Creates a new block of the given type and returns a reference to it
 * \param block_type  The type of block you want to create.  This value is case sensitive and spelling is important.  Ex. NiNode, NiTriShapeData, NiParticleSystemController, etc.
 * \return This function will return a newly created block of the requested type.  Beware, if the block type is unrecognized, this function will return a featureless block with whatever you sent it as the type.
 * 
 * <b>Example:</b> 
 * \code
 * blk_ref my_block = CreateBlock("NiNode");
 * \endcode
 * 
 * <b>In Python:</b>
 * \code
 * my_block = CreateBlock("NiNode")
 * \endcode
 * 
 * sa BlocksInMemory
 */
blk_ref CreateBlock( string block_type );

/*!
 * Returns the current total number of blocks in memory.
 * \return Returns the number of blocks currently in memory.  This includes both blocks loading from nif files and blocks created by the user.  Reference counting is used to ensure that multiple copies of the same block are not created, and blocks that are no longer being used are automatically freed.
 * 
 * <b>Example:</b> 
 * \code
 * unsigned int block_count = BlocksInMemory();
 * \endcode
 * 
 * <b>In Python:</b>
 * \code
 * block_count = BlocksInMemory()
 * \endcode
 * 
 * \sa CreateBlock
 */
unsigned int BlocksInMemory();

//--Query Functions--//
// These are shorthands for using QueryInterface, and required for scripting languages

/*!  A convenience function equivalent to calling IBlock::QueryInterface( ID_SHAPE_DATA ).  It queries the block for an IShapeData interface, and returns a pointer to it if it is present.  Otherwise it returns zero.  In other words, it asks a block if it has the IShapeData interface available.
 * \param block The block to query the interface from.
 * \return If the given block implements the IShapeData interface, a pointer to this interface is returned.  Otherwise the function returns zero  a null pointer.
 * 
 * <b>Example:</b> 
 * \code
 * blk_ref my_block = ReadNifTree("test_in.nif");
 * IShapeData * shape_data = QueryShapeData(my_block);
 * \endcode
 * 
 * <b>In Python:</b>
 * \code
 * my_block = ReadNifTree("test_in.nif")
 * shape_data = QueryShapeData(my_block);
 * \endcode
 * 
 * \sa IBlock::QueryInterface
 */
IShapeData * QueryShapeData( blk_ref & block );
IShapeData const * QueryShapeData( blk_ref const & block );

/*!  A convenience function equivalent to calling IBlock::QueryInterface( ID_TRI_SHAPE_DATA ).  It queries the block for an ITriShapeData interface, and returns a pointer to it if it is present.  Otherwise it returns zero.  In other words, it asks a block if it has the ITriShapeData interface available.
 * \param block The block to query the interface from.
 * \return If the given block implements the ITriShapeData interface, a pointer to this interface is returned.  Otherwise the function returns zero  a null pointer.
 * 
 * <b>Example:</b> 
 * \code
 * blk_ref my_block = ReadNifTree("test_in.nif");
 * ITriShapeData * tri_shape_data = QueryTriShapeData(my_block);
 * \endcode
 * 
 * <b>In Python:</b>
 * \code
 * my_block = ReadNifTree("test_in.nif")
 * tri_shape_data = QueryTriShapeData(my_block);
 * \endcode
 * 
 * \sa IBlock::QueryInterface
 */
ITriShapeData * QueryTriShapeData( blk_ref & block );
ITriShapeData const * QueryTriShapeData( blk_ref const & block );

/*!  A convenience function equivalent to calling IBlock::QueryInterface( ID_SKIN_DATA ).  It queries the block for an ISkinData interface, and returns a pointer to it if it is present.  Otherwise it returns zero.  In other words, it asks a block if it has the ISkinData interface available.
 * \param block The block to query the interface from.
 * \return If the given block implements the ISkinData interface, a pointer to this interface is returned.  Otherwise the function returns zero  a null pointer.
 * 
 * <b>Example:</b> 
 * \code
 * blk_ref my_block = ReadNifTree("test_in.nif");
 * ISkinData * skin_data = QuerySkinData(my_block);
 * \endcode
 * 
 * <b>In Python:</b>
 * \code
 * my_block = ReadNifTree("test_in.nif")
 * skin_data = QuerySkinData(my_block);
 * \endcode
 * 
 * \sa IBlock::QueryInterface
 */
ISkinData * QuerySkinData( blk_ref & block );
ISkinData const * QuerySkinData( blk_ref const & block );

/*!  A convenience function equivalent to calling IBlock::QueryInterface( ID_NODE).  It queries the block for an INode interface, and returns a pointer to it if it is present.  Otherwise it returns zero.  In other words, it asks a block if it has the INode interface available.
 * \param block The block to query the interface from.
 * \return If the given block implements the INode interface, a pointer to this interface is returned.  Otherwise the function returns zero  a null pointer.
 * 
 * <b>Example:</b> 
 * \code
 * blk_ref my_block = ReadNifTree("test_in.nif");
 * INode * node = QueryNode(my_block);
 * \endcode
 * 
 * <b>In Python:</b>
 * \code
 * my_block = ReadNifTree("test_in.nif")
 * node = QueryNode(my_block);
 * \endcode
 * 
 * \sa IBlock::QueryInterface
 */
INode * QueryNode( blk_ref & block );
INode const * QueryNode( blk_ref const & block );

/*!  A convenience function equivalent to calling IBlock::QueryInterface( ID_KEYFRAME_DATA ).  It queries the block for an IKeyframeData interface, and returns a pointer to it if it is present.  Otherwise it returns zero.  In other words, it asks a block if it has the IKeyframeData interface available.
 * \param block The block to query the interface from.
 * \return If the given block implements the IKeyframeData interface, a pointer to this interface is returned.  Otherwise the function returns zero  a null pointer.
 * 
 * <b>Example:</b> 
 * \code
 * blk_ref my_block = ReadNifTree("test_in.nif");
 * IKeyframeData * keyframe_data = QueryKeyframeData(my_block);
 * \endcode
 * 
 * <b>In Python:</b>
 * \code
 * my_block = ReadNifTree("test_in.nif")
 * keyframe_data = QueryKeyframeData(my_block);
 * \endcode
 * 
 * \sa IBlock::QueryInterface
 */
IKeyframeData * QueryKeyframeData( blk_ref & block );
IKeyframeData const * QueryKeyframeData( blk_ref const & block );

/*!  A convenience function equivalent to calling IBlock::QueryInterface( ID_TEXT_KEY_EXTRA_DATA ).  It queries the block for an ITextKeyExtraData interface, and returns a pointer to it if it is present.  Otherwise it returns zero.  In other words, it asks a block if it has the ITextKeyExtraData interface available.
 * \param block The block to query the interface from.
 * \return If the given block implements the ITextKeyExtraData interface, a pointer to this interface is returned.  Otherwise the function returns zero  a null pointer.
 * 
 * <b>Example:</b> 
 * \code
 * blk_ref my_block = ReadNifTree("test_in.nif");
 * ITextKeyExtraData * text_key_extra_data = QueryTextKeyExtraData(my_block);
 * \endcode
 * 
 * <b>In Python:</b>
 * \code
 * my_block = ReadNifTree("test_in.nif")
 * text_key_extra_data = QueryTextKeyExtraData(my_block);
 * \endcode
 * 
 * \sa IBlock::QueryInterface
 */
ITextKeyExtraData * QueryTextKeyExtraData ( blk_ref & block );
ITextKeyExtraData const * QueryTextKeyExtraData ( blk_ref const & block );

/*!  A convenience function equivalent to calling IBlock::QueryInterface( ID_MORPH_DATA ).  It queries the block for an IMorphData interface, and returns a pointer to it if it is present.  Otherwise it returns zero.  In other words, it asks a block if it has the IMorphData interface available.
 * \param block The block to query the interface from.
 * \return If the given block implements the IMorphData interface, a pointer to this interface is returned.  Otherwise the function returns zero  a null pointer.
 * 
 * <b>Example:</b> 
 * \code
 * blk_ref my_block = ReadNifTree("test_in.nif");
 * IMorphData * morph_data = QueryMorphData(my_block);
 * \endcode
 * 
 * <b>In Python:</b>
 * \code
 * my_block = ReadNifTree("test_in.nif")
 * morph_data = QueryMorphData(my_block);
 * \endcode
 * 
 * \sa IBlock::QueryInterface
 */
IMorphData * QueryMorphData ( blk_ref & block );
IMorphData const * QueryMorphData ( blk_ref const & block );

/*!  A convenience function equivalent to calling IBlock::QueryInterface( ID_TRI_STRIPS_DATA ).  It queries the block for an ITriStripsData interface, and returns a pointer to it if it is present.  Otherwise it returns zero.  In other words, it asks a block if it has the ITriStripsData interface available.
 * \param block The block to query the interface from.
 * \return If the given block implements the ITriStripsData interface, a pointer to this interface is returned.  Otherwise the function returns zero  a null pointer.
 * 
 * <b>Example:</b> 
 * \code
 * blk_ref my_block = ReadNifTree("test_in.nif");
 * ITriStripsData * tri_strips_data = QueryTriStripsData(my_block);
 * \endcode
 * 
 * <b>In Python:</b>
 * \code
 * my_block = ReadNifTree("test_in.nif")
 * tri_strips_data = QueryTriStripsData(my_block);
 * \endcode
 * 
 * \sa IBlock::QueryInterface
 */
ITriStripsData * QueryTriStripsData ( blk_ref & block );
ITriStripsData const * QueryTriStripsData ( blk_ref const & block );

/*!  A convenience function equivalent to calling IBlock::QueryInterface( ID_COLOR_DATA ).  It queries the block for an IColorData interface, and returns a pointer to it if it is present.  Otherwise it returns zero.  In other words, it asks a block if it has the IColorData interface available.
 * \param block The block to query the interface from.
 * \return If the given block implements the IColorData interface, a pointer to this interface is returned.  Otherwise the function returns zero  a null pointer.
 * 
 * <b>Example:</b> 
 * \code
 * blk_ref my_block = ReadNifTree("test_in.nif");
 * IColorData * color_data = QueryColorData(my_block);
 * \endcode
 * 
 * <b>In Python:</b>
 * \code
 * my_block = ReadNifTree("test_in.nif")
 * color_data = QueryColorData(my_block);
 * \endcode
 * 
 * \sa IBlock::QueryInterface
 */
IColorData * QueryColorData ( blk_ref & block );
IColorData const * QueryColorData ( blk_ref const & block );

/*!  A convenience function equivalent to calling IBlock::QueryInterface( ID_FLOAT_DATA ).  It queries the block for an IFloatData interface, and returns a pointer to it if it is present.  Otherwise it returns zero.  In other words, it asks a block if it has the IFloatData interface available.
 * \param block The block to query the interface from.
 * \return If the given block implements the IFloatData interface, a pointer to this interface is returned.  Otherwise the function returns zero  a null pointer.
 * 
 * <b>Example:</b> 
 * \code
 * blk_ref my_block = ReadNifTree("test_in.nif");
 * IFloatData * float_data = QueryFloatData(my_block);
 * \endcode
 * 
 * <b>In Python:</b>
 * \code
 * my_block = ReadNifTree("test_in.nif")
 * float_data = QueryFloatData(my_block);
 * \endcode
 * 
 * \sa IBlock::QueryInterface
 */
IFloatData * QueryFloatData ( blk_ref & block );
IFloatData const * QueryFloatData ( blk_ref const & block );

/*!  A convenience function equivalent to calling IBlock::QueryInterface( ID_POS_DATA ).  It queries the block for an IPosData interface, and returns a pointer to it if it is present.  Otherwise it returns zero.  In other words, it asks a block if it has the IPosData interface available.
 * \param block The block to query the interface from.
 * \return If the given block implements the IPosData interface, a pointer to this interface is returned.  Otherwise the function returns zero  a null pointer.
 * 
 * <b>Example:</b> 
 * \code
 * blk_ref my_block = ReadNifTree("test_in.nif");
 * IPosData * pos_data = QueryPosData(my_block);
 * \endcode
 * 
 * <b>In Python:</b>
 * \code
 * my_block = ReadNifTree("test_in.nif")
 * pos_data = QueryPosData(my_block);
 * \endcode
 * 
 * \sa IBlock::QueryInterface
 */
IPosData * QueryPosData ( blk_ref & block );
IPosData const * QueryPosData ( blk_ref const & block );

//--Simple Structures--//

/*! Stores 2D texture coordinates  two floating point variables, u and v. */
struct TexCoord {
	float u; /*!< The U value in this coordinate pair. */ 
	float v; /*!< The V value in this coordinate pair. */ 

	/*! Default constructor	*/
	TexCoord() {}

	/*! This constructor can be used to set all values in this structure during initialization
	 * \param u The value to set U to.
	 * \param v The value to set V to.
	 */
	TexCoord(float u, float v) {
		this->u = u;
		this->v = v;
	}

	/*! This function can be used to set all values in the structure at the same time.
	 * \param u The value to set U to.
	 * \param v The value to set V to.
	 */
	void Set(float u, float v) {
		this->u = u;
		this->v = v;
	}
};

/*! Represents a triangle face formed between three vertices referenced by number */
struct Triangle {
	short v1; /*!< The index of the first vertex. */ 
	short v2; /*!< The index of the second vertex. */ 
	short v3; /*!< The index of the third vertex. */ 

	/*! Default constructor */
	Triangle() {}

	/*! This constructor can be used to set all values in this structure during initialization
	 * \param v1 The index of the first vertex.
	 * \param v2 The index of the second vertex.
	 * \param v3 The index of the third vertex.
	 */
	Triangle(short v1, short v2, short v3) {
		this->v1 = v1;
		this->v2 = v2;
		this->v3 = v3;
	}

	/*! This function can be used to set all values in the structure at the same time.
	 * \param v1 The index of the first vertex.
	 * \param v2 The index of the second vertex.
	 * \param v3 The index of the third vertex.
	 */
	void Set(short v1, short v2, short v3) {
		this->v1 = v1;
		this->v2 = v2;
		this->v3 = v3;
	}
};

/*!Represents a position or direction in 3D space*/
struct Vector3 {
	float x; /*!< The X component of this vector. */ 
	float y; /*!< The Y component of this vector. */ 
	float z; /*!< The Z component of this vector. */ 

	/*!Default constructor.*/
	Vector3() {}

	/*! This constructor can be used to set all values in this structure during initialization
	 * \param x The value to set X to.
	 * \param y The value to set Y to.
	 * \param z The value to set Z to.
	 */
	Vector3(float x, float y, float z) {
		this->x = x;
		this->y = y;
		this->z = z;
	}

	/*! This function can be used to set all values in the structure at the same time.
	 * \param x The value to set X to.
	 * \param y The value to set Y to.
	 * \param z The value to set Z to.
	 */
	void Set(float x, float y, float z) {
		this->x = x;
		this->y = y;
		this->z = z;
	}
};

/* Stores two floating point numbers.  Used as a row of a Matrix22 */
struct Float2 {
	float data[2]; /*!< The two floating point numbers stored as an array. */ 
	
	/*! The bracket operator makes it possible to use this structure like a C++ array.
	 * \param n The index into the data array.  Should be 0 or 1.
	 * \return The value at the given array index by reference so it can be read or set via the bracket operator.
	 */
	float & operator[](int n) {
		return data[n];
	}
	float operator[](int n) const {
		return data[n];
	}

	/*! Default constructor. */
	Float2() {}

	/*! This constructor can be used to set all values in this structure during initialization
	 * \param f1 The value to set the first floating point number to.
	 * \param f2 The value to set the second floating point number to.
	 */
	Float2( float f1, float f2 ) {
		data[0] = f1;
		data[1] = f2;
	}

	/*! This function can be used to set all values in the structure at the same time.
	 * \param f1 The value to set the first floating point number to.
	 * \param f2 The value to set the second floating point number to.
	 */
	void Set( float f1, float f2 ) {
		data[0] = f1;
		data[1] = f2;
	}

	//Python Operator Overloads
	float __getitem__(int n) {
		if (n > 1 || n < 0)
			throw std::out_of_range("Index out of range for MatrixRow3");
        return data[n];
    }
	void __setitem__(int n, float value) {
		if (n > 1 || n < 0)
			throw std::out_of_range("Index out of range for MatrixRow3");
		data[n] = value;
	}
};

/*! Stores a 2 by 2 matrix used for bump maps. */
struct Matrix22 {
	Float2 rows[2];  /*!< The two rows of Float2 structures which hold two floating point numbers each. */ 
	
	/*! The bracket operator makes it possible to use this structure like a 2x2 C++ array.
	 * \param n The index into the row array.  Should be 0 or 1.
	 * \return The Float2 structure for the given row index by reference so it can be read or set via the bracket operator.
	 */
	Float2 & operator[](int n) {
		return rows[n];
	}
	const Float2 & operator[](int n) const {
		return rows[n];
	}

	/*! Default Constructor */
	Matrix22() {}

	/*! This constructor can be used to set all values in this matrix during initialization
	 * \param m11 The value to set at row 1, column 1.
	 * \param m12 The value to set at row 1, column 2.
	 * \param m21 The value to set at row 2, column 1.
	 * \param m22 The value to set at row 2, column 2.
	 */
	Matrix22(
		float m11, float m12,
		float m21, float m22
	) {
		rows[0][0] = m11; rows[0][1] = m12;
		rows[1][0] = m21; rows[1][1] = m22;
	}

	/*! This function can be used to set all values in this matrix at the same time.
	 * \param m11 The value to set at row 1, column 1.
	 * \param m12 The value to set at row 1, column 2.
	 * \param m21 The value to set at row 2, column 1.
	 * \param m22 The value to set at row 2, column 2.
	 */
	void Set(
		float m11, float m12,
		float m21, float m22
	) {
		rows[0][0] = m11; rows[0][1] = m12;
		rows[1][0] = m21; rows[1][1] = m22;
	}

	//Python Operator Overloads
	Float2 & __getitem__(int n) {
		if (n > 1 || n < 0)
			throw std::out_of_range("Index out of range for MatrixRow3");
        return rows[n];
    }
};

/* Stores three floating point numbers.  Used as a row of a Matrix33 and to store the data in attr_vector3 and attr_color3 type attributes. */
struct Float3 {
	float data[3]; /*!< The three floating point numbers stored as an array. */ 

	/*! The bracket operator makes it possible to use this structure like a C++ array.
	 * \param n The index into the data array.  Should be 0, 1, or 2.
	 * \return The value at the given array index by reference so it can be read or set via the bracket operator.
	 */
	float & operator[](int n) {
		return data[n];
	}
	float operator[](int n) const {
		return data[n];
	}

	/*!Default constructor.*/
	Float3() {}

	/*! This constructor can be used to set all values in this structure during initialization
	 * \param f1 The value to set the first floating point number to.
	 * \param f2 The value to set the second floating point number to.
	 * \param f3 The value to set the third floating point number to.
	 */
	Float3( float f1, float f2, float f3 ) {
		data[0] = f1;
		data[1] = f2;
		data[2] = f3;
	}

	/*! This function can be used to set all values in the structure at the same time.
	 * \param f1 The value to set the first floating point number to.
	 * \param f2 The value to set the second floating point number to.
	 * \param f3 The value to set the third floating point number to.
	 */
	void Set( float f1, float f2, float f3 ) {
		data[0] = f1;
		data[1] = f2;
		data[2] = f3;
	}

	//Python Operator Overloads
	float __getitem__(int n) {
		if (n > 2 || n < 0)
			throw std::out_of_range("Index out of range for Float3");
		return data[n];
	}
	void __setitem__(int n, float value) {
		if (n > 2 || n < 0)
			throw std::out_of_range("Index out of range for Float3");
		data[n] = value;
	}
};

/*! Stores a 3 by 3 matrix used for rotation. */
struct Matrix33 {
	Float3 rows[3]; /*!< The three rows of Float3 structures which hold three floating point numbers each. */ 
	
	/*! The bracket operator makes it possible to use this structure like a 3x3 C++ array.
	 * \param n The index into the row array.  Should be 0, 1, or 2.
	 * \return The Float3 structure for the given row index by reference so it can be read or set via the bracket operator.
	 */
	Float3 & operator[](int n) {
		return rows[n];
	}
	const Float3 & operator[](int n) const {
		return rows[n];
	}

	/*! Default constructor. */
	Matrix33() {}

	/*! This constructor can be used to set all values in this matrix during initialization
	 * \param m11 The value to set at row 1, column 1.
	 * \param m12 The value to set at row 1, column 2.
	 * \param m13 The value to set at row 1, column 3.
	 * \param m21 The value to set at row 2, column 1.
	 * \param m22 The value to set at row 2, column 2.
	 * \param m23 The value to set at row 2, column 3.
	 * \param m31 The value to set at row 3, column 1.
	 * \param m32 The value to set at row 3, column 2.
	 * \param m33 The value to set at row 3, column 3.
	 */
	Matrix33(
		float m11, float m12, float m13,
		float m21, float m22, float m23,
		float m31, float m32, float m33
	) {
		rows[0][0] = m11; rows[0][1] = m12; rows[0][2] = m13;
		rows[1][0] = m21; rows[1][1] = m22; rows[1][2] = m23;
		rows[2][0] = m31; rows[2][1] = m32; rows[2][2] = m33;
	}

	/*! This function can be used to set all values in this matrix at the same time.
	 * \param m11 The value to set at row 1, column 1.
	 * \param m12 The value to set at row 1, column 2.
	 * \param m13 The value to set at row 1, column 3.
	 * \param m21 The value to set at row 2, column 1.
	 * \param m22 The value to set at row 2, column 2.
	 * \param m23 The value to set at row 2, column 3.
	 * \param m31 The value to set at row 3, column 1.
	 * \param m32 The value to set at row 3, column 2.
	 * \param m33 The value to set at row 3, column 3.
	 */
	void Set(
		float m11, float m12, float m13,
		float m21, float m22, float m23,
		float m31, float m32, float m33
	) {
		rows[0][0] = m11; rows[0][1] = m12; rows[0][2] = m13;
		rows[1][0] = m21; rows[1][1] = m22; rows[1][2] = m23;
		rows[2][0] = m31; rows[2][1] = m32; rows[2][2] = m33;
	}

	void AsFloatArr( float out[3][3] ) {
		out[0][0] = rows[0][0]; out[0][1] = rows[0][1]; out[0][2] = rows[0][2];
		out[1][0] = rows[1][0]; out[1][1] = rows[1][1]; out[1][2] = rows[1][2];
		out[2][0] = rows[2][0]; out[2][1] = rows[2][1]; out[2][2] = rows[2][2];
	}

	//Python Operator Overloads
	Float3 & __getitem__(int n) {
		if (n > 2 || n < 0)
			throw std::out_of_range("Index out of range for MatrixRow3");
        return rows[n];
    }
};

/* Stores four floating point numbers.  Used as a row of a Matrix44. */
struct Float4 {
	float data[4]; /*!< The four floating point numbers stored as an array. */ 

	/*! The bracket operator makes it possible to use this structure like a C++ array.
	 * \param n The index into the data array.  Should be 0, 1, 2, or 3.
	 * \return The value at the given array index by reference so it can be read or set via the bracket operator.
	 */
	float & operator[](int n) {
		return data[n];
	}
	float operator[](int n) const {
		return data[n];
	}

	/*! Default Constructor.*/
	Float4() {}

	/*! This constructor can be used to set all values in this structure during initialization
	 * \param f1 The value to set the first floating point number to.
	 * \param f2 The value to set the second floating point number to.
	 * \param f3 The value to set the third floating point number to.
	 * \param f4 The value to set the fourth floating point number to.
	 */
	Float4( float f1, float f2, float f3, float f4 ) {
		data[0] = f1;
		data[1] = f2;
		data[3] = f3;
		data[4] = f4;
	}

	/*! This function can be used to set all values in the structure at the same time.
	 * \param f1 The value to set the first floating point number to.
	 * \param f2 The value to set the second floating point number to.
	 * \param f3 The value to set the third floating point number to.
	 * \param f4 The value to set the fourth floating point number to.
	 */
	void Set( float f1, float f2, float f3, float f4 ) {
		data[0] = f1;
		data[1] = f2;
		data[3] = f3;
		data[4] = f4;
	}

	//Python Operator Overloads
	float __getitem__(int n) {
		if (n > 3 || n < 0)
			throw std::out_of_range("Index out of range for Float4");
        return data[n];
    }
	void __setitem__(int n, float value) {
		if (n > 3 || n < 0)
			throw std::out_of_range("Index out of range for Float4");
		data[n] = value;
	}
};

/*! Stores a 4 by 4 matrix used for combined transformations. */
struct Matrix44 {
	Float4 rows[4]; /*!< The three rows of Float3 structures which hold three floating point numbers each. */ 
	
	/*! The bracket operator makes it possible to use this structure like a 4x4 C++ array.
	 * \param n The index into the row array.  Should be 0, 1, 2, or 3.
	 * \return The Float4 structure for the given row index by reference so it can be read or set via the bracket operator.
	 */
	Float4 & operator[](int n) {
		return rows[n];
	}
	Float4 const & operator[](int n) const {
		return rows[n];
	}

	/*! Default constructor. */
	Matrix44() {}

	/*! This constructor can be used to set all values in this matrix during initialization
	 * \param m11 The value to set at row 1, column 1.
	 * \param m12 The value to set at row 1, column 2.
	 * \param m13 The value to set at row 1, column 3.
	 * \param m14 The value to set at row 1, column 4.
	 * \param m21 The value to set at row 2, column 1.
	 * \param m22 The value to set at row 2, column 2.
	 * \param m23 The value to set at row 2, column 3.
	 * \param m24 The value to set at row 2, column 4.
	 * \param m31 The value to set at row 3, column 1.
	 * \param m32 The value to set at row 3, column 2.
	 * \param m33 The value to set at row 3, column 3.
	 * \param m34 The value to set at row 3, column 4.
	 * \param m41 The value to set at row 4, column 1.
	 * \param m42 The value to set at row 4, column 2.
	 * \param m43 The value to set at row 4, column 3.
	 * \param m44 The value to set at row 4, column 4.
	 */
	Matrix44(
		float m11, float m12, float m13, float m14,
		float m21, float m22, float m23, float m24,
		float m31, float m32, float m33, float m34,
		float m41, float m42, float m43, float m44
	) {
		rows[0][0] = m11; rows[0][1] = m12; rows[0][2] = m13; rows[0][3] = m14;
		rows[1][0] = m21; rows[1][1] = m22; rows[1][2] = m23; rows[1][3] = m24;
		rows[2][0] = m31; rows[2][1] = m32; rows[2][2] = m33; rows[2][3] = m34;
		rows[3][0] = m41; rows[3][1] = m42; rows[3][2] = m43; rows[3][3] = m44;
	}

	/*! This function can be used to set all values in this matrix at the same time.
	 * \param m11 The value to set at row 1, column 1.
	 * \param m12 The value to set at row 1, column 2.
	 * \param m13 The value to set at row 1, column 3.
	 * \param m14 The value to set at row 1, column 4.
	 * \param m21 The value to set at row 2, column 1.
	 * \param m22 The value to set at row 2, column 2.
	 * \param m23 The value to set at row 2, column 3.
	 * \param m24 The value to set at row 2, column 4.
	 * \param m31 The value to set at row 3, column 1.
	 * \param m32 The value to set at row 3, column 2.
	 * \param m33 The value to set at row 3, column 3.
	 * \param m34 The value to set at row 3, column 4.
	 * \param m41 The value to set at row 4, column 1.
	 * \param m42 The value to set at row 4, column 2.
	 * \param m43 The value to set at row 4, column 3.
	 * \param m44 The value to set at row 4, column 4.
	 */
	void Set(
		float m11, float m12, float m13, float m14,
		float m21, float m22, float m23, float m24,
		float m31, float m32, float m33, float m34,
		float m41, float m42, float m43, float m44
	) {
		rows[0][0] = m11; rows[0][1] = m12; rows[0][2] = m13; rows[0][3] = m14;
		rows[1][0] = m21; rows[1][1] = m22; rows[1][2] = m23; rows[1][3] = m24;
		rows[2][0] = m31; rows[2][1] = m32; rows[2][2] = m33; rows[2][3] = m34;
		rows[3][0] = m41; rows[3][1] = m42; rows[3][2] = m43; rows[3][3] = m44;
	}

	void AsFloatArr( float out[4][4] ) {
		out[0][0] = rows[0][0]; out[0][1] = rows[0][1]; out[0][2] = rows[0][2]; out[0][3] = rows[0][3];
		out[1][0] = rows[1][0]; out[1][1] = rows[1][1]; out[1][2] = rows[1][2]; out[1][3] = rows[1][3];
		out[2][0] = rows[2][0]; out[2][1] = rows[2][1]; out[2][2] = rows[2][2]; out[2][3] = rows[2][3];
		out[3][0] = rows[3][0]; out[3][1] = rows[3][1]; out[3][2] = rows[3][2]; out[3][3] = rows[3][3];
	}

	//Python Operator Overloads
	Float4 & __getitem__(int n) {
		if (n > 3 || n < 0)
			throw std::out_of_range("Index out of range for Matrix44");
        return rows[n];
    }
};

/*! Stores a color along with alpha translucency */
struct Color4 {
	float r; /*!< The red component of this color.  Should be between 0.0f and 1.0f. */ 
	float g; /*!< The green component of this color.  Should be between 0.0f and 1.0f. */ 
	float b; /*!< The blue component of this color.  Should be between 0.0f and 1.0f. */ 
	float a; /*!< The alpha translucency component of this color.  Should be between 0.0f and 1.0f. */ 

	/*! Default constructor */
	Color4() {}

	/*! This constructor can be used to set all values in this structure during initialization
	 * \param r The value to set the red component of this color to.  Should be between 0.0f and 1.0f.
	 * \param g The value to set the green component of this color to. Should be between 0.0f and 1.0f.
	 * \param b The value to set the blue component of this color to.  Should be between 0.0f and 1.0f.
	 * \param a The value to set the alpha translucency component of this color to.  Should be between 0.0f and 1.0f.
	 */
	Color4(float r, float g, float b, float a = 1.0f) {
		this->r = r;
		this->g = g;
		this->b = b;
		this->a = a;
	}

	/*! This function can be used to set all values in the structure at the same time.
	 * \param r The value to set the red component of this color to.  Should be between 0.0f and 1.0f.
	 * \param g The value to set the green component of this color to. Should be between 0.0f and 1.0f.
	 * \param b The value to set the blue component of this color to.  Should be between 0.0f and 1.0f.
	 * \param a The value to set the alpha translucency component of this color to.  Should be between 0.0f and 1.0f.
	 */
	void Set(float r, float g, float b, float a = 1.0f) {
		this->r = r;
		this->g = g;
		this->b = b;
		this->a = a;
	}
};

/*! Represents a quaternion - a 4D extention of complex numbers used as an alternitive to matrices to represent rotation.*/
struct Quaternion {
	float w; /*!< The W scalar component of this Quaternion. */ 
	float x; /*!< The X vector component of this Quaternion. */ 
	float y; /*!< The Y vector component of this Quaternion. */ 
	float z; /*!< The Z vector component of this Quaternion. */ 

	/*! Default constructor. */
	Quaternion() {}

	/*! This constructor can be used to set all values in this structure during initialization
	 * \param w The value to set the W scalar component of this quaternion to.
	 * \param x The value to set the X vector component of this quaternion to.
	 * \param y The value to set the Y vector component of this quaternion to.
	 * \param z The value to set the Z vector component of this quaternion to.
	 */
	Quaternion(float w, float x, float y, float z) {
		this->w = w;
		this->x = x;
		this->y = y;
		this->z = z;
	}

	/*! This function can be used to set all values in the structure at the same time.
	 * \param w The value to set the W scalar component of this quaternion to.
	 * \param x The value to set the X vector component of this quaternion to.
	 * \param y The value to set the Y vector component of this quaternion to.
	 * \param z The value to set the Z vector component of this quaternion to.
	 */
	void Set(float w, float x, float y, float z) {
		this->w = w;
		this->x = x;
		this->y = y;
		this->z = z;
	}
};

/*! Represents a bounding box - the smallest rectangular shape that a particular node can fit inside. */
struct BoundingBox {
	bool isUsed; /*!< Determines whether this bounding box is used or not.  If this value is true, the other members of this structure are significant.  If false, they are ignored. */ 
	int unknownInt; /*!< An integer whos function is still unknown. */
	Vector3 translation; /*!< The translation vector (position relative to the origin) of this bounding box. */
	Matrix33 rotation; /*!< The rotation of this bounding box expressed as a 3x3 matrix. */
	Vector3 radius; /*!< A vector containing the radius of this box in the direction of each axis, X, Y, and Z. */
};

/*! Holds an integer that may or may not be used. */
struct ConditionalInt {
	bool isUsed; /*!< Determines whether the integer contained within this structure is used or not.  If this value is true, the integer is significant.  If false, it is ignored. */ 
	int unknownInt; /*!< The integer value which may or may not be in use.  Its function is unknown. */ 
};

/*! Represents a texture description that specifies various properties of the texture that it refers to. The NiTextureSource block that this description refers to can be retrieved by calling asLink on the same attribute. */
struct TexDesc {
	/*! Default constructor.  Sets isUsed to false, clampMode to WRAP_S_WRAP_T, filterMode to FILTER_TRILERP, testureSet to 0, PS2_L to zero, PS2_K to 0xFFB5, and unknownShort to 0x0101.*/
	TexDesc() : isUsed(false), clampMode(WRAP_S_WRAP_T), filterMode(FILTER_TRILERP), textureSet(0),  PS2_L(0), PS2_K(0xFFB5), unknownShort(0x0101) {}
	
	bool isUsed; /*!< Determines whether this texture description is used or not.  If this value is true, the other members of this structure are significant.  If false, they are ignored. */ 
	TexClampMode clampMode;  /*!< The texture wraping/clamping mode. */ 
	TexFilterMode filterMode; /*!< The texture filtering mode. */ 
	int textureSet; /*!< Texture set? Usually 0. */ 
	unsigned short PS2_L; /*!< Something to do with Play Station 2 texture filtering.  Usually 0. */ 
	unsigned short PS2_K; /*!< Something to do with Play Station 2 texture filtering.  Usually 0xFFB5. */ 
	short unknownShort;  /*!< An unknown short value. Exists up to version 4.1.0.12 */
	//Unknown Block in version 10.1.0.0 and up
	bool hasUnknownData; /*!< If this is true, the unknown5Floats, unknownInt, unknownFloat1, and unknownFloat2 members are significant.  These properties only exist after version 10.1.0.0. */ 
	float unknown5Floats[5]; /*!< 5 unkown floating point values that exist from version 10.1.0.0 on. */ 
	int unknownInt; /*!< An unknown integer value that exists from version 10.1.0.0 on. */ 
	float unknownFloat1; /*!< An unknown floating point value that exists from version 10.1.0.0 on. */ 
	float unknownFloat2; /*!< An unknown floating point value that exists from version 10.1.0.0 on. */ 
	//Bitmap block - only exists if this texture is in the bitmap slot
	float bmLumaOffset; /*!< The bitmap luma offset.  Unsure of function.  Only exists if this texture is in the bitmap slot. */ 
	float bmLumaScale; /*!< The bitmap luma scale.  Unsure of function.  Only exists if this texture is in the bitmap slot. */ 
	Matrix22 bmMatrix; /*!< The bitmap 2x2 matrix.  Unsure of function.  Only exists if this texture is in the bitmap slot. */ 
};

/*! Stores texture source data.  Specifies where to find the image data; in an external file, or within a NiPixelData block. */
struct TexSource {
	bool useExternal; /*!< Specifies whether to use an external file for the texture or not.  If true, an external file is used.  If false, the image data is stored within a NiPixelData block.  This block can be retrieved by using the asLink function on the same attribute. */ 
	unsigned char unknownByte; /*!< A byte whos function is unknown. */
	string fileName; /*!< The filename of the texture, if stored externally.  Can be an image file such as a TGA, BMP, or DDS file, or another NIF file which stores the image data within a NiPixelData block. */ 
};

/*! Stores one skin weight of a vertex by number. */
struct SkinWeight {
	unsigned short vertexNum; /*!< The vertex number that this weight is for. */
	float vertexWeight; /*!< The amount a particular bone affects the movement of this vertex.  Should be a number between 0.0f and 1.0f. */
};

/*! Stores an animation key and the time in the animation that it takes affect. It is a template class so it can hold any kind of data as different blocks key different sorts of information to the animation timeline.*/
template <class T> 
struct Key {
	float time; /*!< The time on the animation timeline that this keyframe takes affect. */ 
	T data; /*!< The data being keyed to the timeline. */ 
	T forward_tangent; /*!< A piece of data of the same type as is being keyed to the time line used as the forward tangent in quadratic interpolation.  Ignored if key type is set as something else. */ 
	T backward_tangent; /*!< A piece of data of the same type as is being keyed to the time line used as the backward tangent in quadratic interpolation.  Ignored if key type is set as something else. */ 
	float tension; /*!< The amount of tension to use in tension, bias, continuity interpolation.  Ignored if key type is something else.*/
	float bias; /*!< The amount of bias to use in tension, bias, continuity interpolation.  Ignored if key type is something else.*/
	float continuity; /*!< The amount of continuity to use in tension, bias, continuity interpolation.  Ignored if key type is something else.*/
};

//--Main Interfaces--//

/*! The base interface for all NIF blocks. 
 * \sa blk_ref, CreateBlock
 */
class IBlock{
public:
	IBlock( ){}
	virtual ~IBlock() {}

	/*! Returns the last known block number.  Block numbers are set automatically when a Nif file is first read and again each time a tree is written.
	 * \return The last known block number.  Will not update until the block is written to a file.
	 * \sa WriteNifTree, blk_ref::get_index
	 */
	virtual int GetBlockNum() const = 0;

	/*!
	 * Used to retrieve the first parent that was linked to this block.
	 * \return A block reference to the first parent that was linked to this block.  If there is no parent, a null reference is returned.
	 * 
	 * <b>Example:</b> 
	 * \code
	 * blk_ref my_block = ReadNifTree("test_in.nif");
	 * blk_ref parent_blk = my_block->GetParent();
	 * \endcode
	 * 
	 * <b>In Python:</b>
	 * \code
	 * my_block = ReadNifTree("test_in.nif")
	 * parent_blk =  block.GetParent()
	 * \endcode
	 * 
	 * \sa IAttr::Set(blk_ref const &), IAttr::AddLink, IAttr::AddLinks, IAttr::RemoveLinks, IAttr::ClearLinks
	 */
	virtual blk_ref GetParent() const = 0;

	/*!
	 * Summarizes the information contained in this block in English.
	 * \return A string containing a summary of the information within the block in English.  This is the function that Niflyze calls to generate its analysis, so the output is the same.
	 * 
	 * <b>Example:</b> 
	 * \code
	 * blk_ref my_block = ReadNifTree("test_in.nif");
	 * cout << my_block->asString();
	 * \endcode
	 * 
	 * <b>In Python:</b>
	 * \code
	 * my_block = ReadNifTree("test_in.nif")
	 * print block.asString()
	 * \endcode
	 * 
	 * \sa IAttr::asString, SetVerboseMode
	 */
	virtual string asString() const = 0;

	/*!
	 * Used to determine a blocks type.  These type strings are the same as the class names of the blocks in the <a href = "http://niftools.sourceforge.net/docsys/">NIF File Format Browser</a>.
	 * \return A string containing the type of the block.  Ex. NiNode, NiTriShapeData, NiParticleSystemController, etc.
	 * 
	 * <b>Example:</b> 
	 * \code
	 * blk_ref my_block = ReadNifTree("test_in.nif");
	 * cout << my_block->GetBlockType();
	 * \endcode
	 * 
	 * <b>In Python:</b>
	 * \code
	 * my_block = ReadNifTree("test_in.nif")
	 * print block.GetBlockType()
	 * \endcode
	 * 
	 * \sa CreateBlock
	 */
	virtual string GetBlockType() const = 0;

	/*!
	 * Used to determine whether this block is controllable - i.e. a controller can animate it.  This means it decends from the AControllable class in the <a href = "http://niftools.sourceforge.net/docsys/">NIF File Format Browser</a>.
	 * \return True if the block is controllable.  False otherwise.
	 * \sa IBlock::IsController, QueryNode
	 */
	virtual bool IsControllable() const = 0;

	/*!
	 * Used to determine whether this block is an animation controller.  This means it decends from the AController class in the <a href = "http://niftools.sourceforge.net/docsys/">NIF File Format Browser</a>.
	 * \return True if the block is an animation controller.  False otherwise.
	 * \sa IBlock::IsControllable, QueryNode
	 */
	virtual bool IsController() const = 0;

	//--Attribute Functions--//

	/*!
	 * Used to get an attribute reference from a block by name.
	 * \param attr_name The name of the attribute you are requesting.  Ex. Translation, Bounding Box, Unknown Int 1, ec.
	 * \return If the attribute exits in the block, a reference to it is returned.  Otherwise a null reference is returned.
	 * 
	 * <b>Example:</b> 
	 * \code
	 * blk_ref my_block = ReadNifTree("test_in.nif");
	 * attr_ref my_attr = my_block["NiNode"]; // Shortcut using [] operator
	 * my_attr = my_block->GetAttr("NiNode"); // Same thing but longer
	 * \endcode
	 * 
	 * <b>In Python:</b>
	 * \code
	 * my_block = ReadNifTree("test_in.nif")
	 * my_attr = my_block["NiNode"] # Shortcut using __getitem__ operator
	 * my_attr = my_block.GetAttr("NiNode") # Same thing but longer
	 * \endcode
	 * 
	 * \sa IBlock::GetAttrs, attr_ref, blk_ref::operator[]
	 */
	virtual attr_ref GetAttr(string const & attr_name) const = 0;

	/*!
	 * Used to get a vector holding a reference to every all attribute contained in a block.
	 * \return A vector of references for the complete list of attributes contained within a block.  The list can then be used to visit each attribute.
	 * 
	 * <b>Example:</b> 
	 * \code
	 * blk_ref my_block = ReadNifTree("test_in.nif");
	 * vector<attr_ref> attr_list = my_block->GetAttrs();
	 * \endcode
	 * 
	 * <b>In Python:</b>
	 * \code
	 * my_block = ReadNifTree("test_in.nif")
	 * attr_list = my_block.GetAttrs();
	 * \endcode
	 * 
	 * \sa IBlock::GetAttr, attr_ref
	 */
	virtual vector<attr_ref> GetAttrs() const = 0;

	//--Link Function--//

	/*!
	 * Used to retrieve all blocks that the current block is linked to through <i>all</i> attributes.
	 * \return A list of references to blocks that this attribute links its owner block to.
	 * 
	 * <b>Example:</b> 
	 * \code
	 * blk_ref my_block = ReadNifTree("test_in.nif");
	 * list<blk_ref> attr_list = my_block->GetLinks();
	 * \endcode
	 * 
	 * <b>In Python:</b>
	 * \code
	 * my_block = ReadNifTree("test_in.nif")
	 * attr_list = my_block.GetLinks()
	 * \endcode
	 * 
	 * \sa IAttr::asLink, IAttr::asLinkList, IAttr::FindLink
	 */
	virtual list<blk_ref> GetLinks() const = 0;

	//--To check for specialized Interfaces--//

	/*!
	 * Used to query for advanced block interfaces such as INode and IShapeData.  This function is not availiable from Python; you will need to use one of the many shorthand Query functions such as QueryNode and QueryShapeData.
	 * \param id The ID constant of the interface you're querying, such as ID_NODE or ID_SHAPE_DATA.
	 * \return If this block implements the requested interface, a pointer to it is returned.  Otherwise zero is returned - a null pointer.
	 * 
	 * <b>Example:</b> 
	 * \code
	 * block = CreateBlock("NiNode");
	 * INode * node = (INode*)block->QueryInterface(ID_NODE);
	 * \endcode
	 * 
	 * \sa QueryShapeData, QueryTriShapeData, QuerySkinData, QueryNode, QueryKeyframeData, QueryTextKeyExtraData, QueryMorphData, QueryTriStripsData, QueryColorData, QueryFloatData, QueryPosData
	 */
	virtual void * QueryInterface( int id ) = 0;
	virtual void const * QueryInterface( int id ) const = 0;
	
	// Python Operator Overloads
	string __str__() const {
		return asString();
	};
	
protected:
	friend class blk_ref;
	virtual void AddRef() = 0;
	virtual void SubtractRef() = 0;
};

/*! The base interface for all NIF attributes.
 * \sa attr_ref
 */
class IAttr {
public:
	IAttr() {}
	virtual ~IAttr() {}

	/*!
	 * Used to determine the type of an attribute at runtime.
	 * \return An AttrType constant reffering to the type of the attribute. Ex. attr_float, attr_texture, attr_byte, etc.
	 * 
	 * <b>Example:</b> 
	 * \code
	 * blk_ref my_block = ReadNifTree("test_in.nif");
	 * AttrType attr_type = my_block["Scale"]->GetType();
	 * \endcode
	 * 
	 * <b>In Python:</b>
	 * \code
	 * my_block = ReadNifTree("test_in.nif")
	 * attr_type = my_block["Scale"].GetType()
	 * \endcode
	 * 
	 * \sa IBlock::GetBlockType
	 */
	virtual AttrType GetType() const = 0;

	/*!
	 * Used to determine an attributes friendly name.
	 * \return A string that contains the friendly name of the attribute.  This is the same name used with the IBlock::GetAttr function to retrieve an attribute. Ex. Scale, Texture, Skeleton Root, etc.
	 * 
	 * <b>Example:</b> 
	 * \code
	 * blk_ref my_block = ReadNifTree("test_in.nif");
	 * vector<attr_ref> attr_list = my_block->GetAttrs();
	 * for ( int i = 0; i < attr_list.size(); ++i ) {
     *	cout << attr_list[i]->GetName() << endl;
	 * }
	 * \endcode
	 * 
	 * <b>In Python:</b>
	 * \code
	 * my_block = ReadNifTree("test_in.nif")
	 * attr_list = my_block.GetAttrs()
	 * for attr in attr_list:
     *	print attr.GetName()
	 * \endcode
	 * 
	 * \sa IBlock::GetBlockType
	 */
	virtual string GetName() const = 0;

	virtual void Read( ifstream& in, unsigned int version ) = 0;
	virtual void Write( ofstream& out, unsigned int version ) const = 0;

	//--Getters--//

	/*!
	 * Used to get a copy of the integer value stored in an attribute.  Raises an exception if the attribute does not store this type of value.
	 * \return A copy of the integer value stored in this attribute.
	 */
	virtual int asInt() const = 0;

	/*!
	 * Used to get a copy of the floating point value stored in an attribute.  Raises an exception if the attribute does not store this type of value.
	 * \return A copy of the floating point value stored in this attribute.
	 * 
	 * <b>Example:</b> 
	 * \code
	 * blk_ref my_block = ReadNifTree("test_in.nif");
	 * float scale = my_block["Scale"]; // Shortcut using overloaded float operator
	 * scale = my_block["Scale"]->asFloat();  // Same thing but longer
	 * \endcode
	 * 
	 * <b>In Python:</b>
	 * \code
	 * my_block = ReadNifTree("test_in.nif")
	 * scale = my_block["Scale"].asFloat() # shortcut not available due to Python assignment semantics
	 * \endcode
	 */
	virtual float asFloat() const = 0;

	/*!
	 * Used to get a copy of the Float3 structure stored in an attribute.  Raises an exception if the attribute does not store this type of value.
	 * \return A copy of the Float3 structure stored in this attribute.
	 */
	virtual Float3 asFloat3() const = 0;

	/*!
	 * Used to get a copy of the text string stored in an attribute.  Raises an exception if the attribute does not store this type of value.
	 * \return A copy of the text string stored in this attribute.
	 */
	virtual string asString() const = 0;

	/*!
	 * Used to get a copy of the Matrix33 structure stored in an attribute.  Raises an exception if the attribute does not store this type of value.
	 * \return A copy of the Matrix33 structure stored in this attribute.
	 */
	virtual Matrix33 asMatrix33() const = 0;

	/*!
	 * Used to get a block reference to the block linked through this attribute to its owner block.  Raises an exception if the attribute does not store this type of value.
	 * \return A block reference to the block linked through this attribute to its owner block.  If no block is linked through this attribute, a null reference is returned.
	 */
	virtual blk_ref asLink() const = 0;

	/*!
	 * Used to get a copy of the TexSource structure stored in an attribute.  Raises an exception if the attribute does not store this type of value.
	 * \return A copy of the TexSource structure stored in this attribute.
	 */
	virtual TexSource asTexSource() const = 0;

	/*!
	 * Used to get a copy of the BoundingBox structure stored in an attribute.  Raises an exception if the attribute does not store this type of value.
	 * \return A copy of the BoundingBox structure stored in this attribute.
	 */
	virtual BoundingBox asBoundingBox() const = 0;

	/*!
	 * Used to get a copy of the ConditionalInt structure stored in an attribute.  Raises an exception if the attribute does not store this type of value.
	 * \return A copy of the ConditionalInt structure stored in this attribute.
	 */
	virtual ConditionalInt asConditionalInt() const = 0;

	/*!
	 * Used to get a copy of the TexDesc structure stored in an attribute.  Raises an exception if the attribute does not store this type of value.
	 * \return A copy of the TexDesc structure stored in this attribute.
	 */
	virtual TexDesc asTexDesc() const = 0;

	/*!
	 * Used to retrieve a list of all the blocks linked through this attribute to its owner block.  Raises an exception if the attribute does not store this type of value.
	 * \return A list of all the blocks linked through this attribute to its owner block. The list will be empty if no blocks are linked through this attribute.
	 */
	virtual list<blk_ref> asLinkList() const = 0;

	//--Setters--//

	/*!
	 * Used to change the value stored in an attribute.  Raises an exception if the attribute does not store the type of value that the Set function is called on.
	 * \param val This value will be copied into the attribute and stored.
	 */
	virtual void Set( int val ) = 0;

	/*!
	 * Used to change the value stored in an attribute.  Raises an exception if the attribute does not store the type of value that the Set function is called on.
	 * \param val This value will be copied into the attribute and stored.
	 * 
	 * <b>Example:</b> 
	 * \code
	 * blk_ref my_block = ReadNifTree("test_in.nif");
	 * my_block["Scale"] = 0.7f; //Shortcut using overloaded = operator
	 * my_block["Scale"]->Set(0.7f);  //Same thing but longer
	 * \endcode
	 * 
	 * <b>In Python:</b>
	 * \code
	 * my_block = ReadNifTree("test_in.nif")
	 * my_block["Scale"] = 0.7 # Shortcut using overloaded __setitem__ operator
	 * my_block["Scale"].Set(0.7) # Same thing but longer
	 * \endcode
	 */
	virtual void Set( float val ) = 0;

	/*!
	 * Used to change the value stored in an attribute.  Raises an exception if the attribute does not store the type of value that the Set function is called on.
	 * \param val This value will be copied into the attribute and stored.
	 */
	virtual void Set( Float3 const & val ) = 0;

	/*!
	 * Used to change the value stored in an attribute.  Raises an exception if the attribute does not store the type of value that the Set function is called on.
	 * \param val This value will be copied into the attribute and stored.
	 */
	virtual void Set( string const & val ) = 0;

	/*!
	 * Used to change the value stored in an attribute.  Raises an exception if the attribute does not store the type of value that the Set function is called on.
	 * \param val This value will be copied into the attribute and stored.
	 */
	virtual void Set( Matrix33 const & val ) = 0;

	/*!
	 * Used to retrieve the block that the current block is linked to through this attribute.  Raises an exception if the attribute does not store the type of value that the Set function is called on.
	 * \param val This is the block to be linked to the block which owns this attribute. Passing a null block reference will break the link.
	 *
	 * <b>Example:</b> 
	 * \code
	 * blk_ref my_block = ReadNifTree("test_in.nif");
	 * blk_ref new_controller = CreateBlock("NiKeyframeController");
	 * my_block["Controller"] = new_controller; //Shortcut using overloaded = operator
	 * my_block["Controller"]->Set(new_controller);  //Same thing but longer
	 * \endcode
	 * 
	 * <b>In Python:</b>
	 * \code
	 * my_block = ReadNifTree("test_in.nif")
	 * new_controller = CreateBlock("NiKeyframeController")
	 * my_block["Controller"].Set(new_controller) #shortcuts not available
	 * \endcode
	 */
	virtual void Set( blk_ref const & val ) = 0;

	/*!
	 * Used to change the value stored in an attribute.  Raises an exception if the attribute does not store the type of value that the Set function is called on.
	 * \param val This value will be copied into the attribute and stored.
	 */
	virtual void Set( TexSource const & val ) = 0;

	/*!
	 * Used to change the value stored in an attribute.  Raises an exception if the attribute does not store the type of value that the Set function is called on.
	 * \param val This value will be copied into the attribute and stored.
	 */
	virtual void Set( BoundingBox const & val ) = 0;

	/*!
	 * Used to change the value stored in an attribute.  Raises an exception if the attribute does not store the type of value that the Set function is called on.
	 * \param val This value will be copied into the attribute and stored.
	 */
	virtual void Set( ConditionalInt const & val ) = 0;

	/*!
	 * Used to change the value stored in an attribute.  Raises an exception if the attribute does not store the type of value that the Set function is called on.
	 * \param val This value will be copied into the attribute and stored.
	 */
	virtual void Set( TexDesc const & val ) = 0;

	//--Link functions--//

	/*!
	 * Used to determine whether this attribute can store links or not.  Several attribute types can store links, but with this function you can quickly tell if any links are within an attribute.
	 * \return True if this attribute can store one or more links. False if it cannot store any links.
	 * \sa IAttr:Set( blk_ref const & ), IAttr::asLink, IAttr::asLinkList, IAttr::AddLink, IAttr::AddLinks, IAttr::ClearLinks, IAttr::RemoveLinks, IAttr::FindLink
	 */
	virtual bool HasLinks() const = 0;

	/*!
	 * Used to add a link to this attributes list of blocks that are linked to its owner block.
	 * \param block A reference to the block to add to the list of blocks that are linked to this attributes owner block.
	 *
	 * <b>Example:</b> 
	 * \code
	 * blk_ref my_block = ReadNifTree("test_in.nif");
	 * blk_ref block2 = CreateBlock("NiNode");
	 * my_block["Children"]->AddLink(block2);
	 * \endcode
	 * 
	 * <b>In Python:</b>
	 * \code
	 * my_block = ReadNifTree("test_in.nif")
	 * block2 = CreateBlock("NiNode")
	 * my_block["Children"].AddLink(block2);
	 * \endcode
	 * \sa IAttr:Set( blk_ref const & ), IAttr::HasLinks, IAttr::asLink, IAttr::asLinkList, IAttr::AddLinks, IAttr::ClearLinks, IAttr::RemoveLinks, IAttr::FindLink
	 */
	virtual void AddLink( blk_ref const & block ) = 0;

	/*!
	 * Used to add several links at once to this attributes list of blocks that are linked to its owner block.
	 * \param new_links A list of block references to add to the list of blocks that are linked to this attributes owner block.
	 *
	 * <b>Example:</b> 
	 * \code
	 * blk_ref my_block = ReadNifTree("test_in.nif");
	 * list<blk_ref> blk_list;
	 * blk_list.push_back( CreateBlock("NiNode") );
	 * blk_list.push_back( CreateBlock("NiTriShape") );
	 * my_block["Children"]->AddLinks(blk_list);
	 * \endcode
	 * 
	 * <b>In Python:</b>
	 * \code
	 * my_block = ReadNifTree("test_in.nif")
	 * blk_list = ( CreateBlock("NiNode"), CreateBlock("NiTriShape") )
	 * my_block["Children"].AddLinks(blk_list)
	 * \endcode
	 * \sa IAttr:Set( blk_ref const & ), IAttr::HasLinks, IAttr::asLink, IAttr::asLinkList, IAttr::AddLink, IAttr::ClearLinks, IAttr::RemoveLinks, IAttr::FindLink
	 */
	virtual void AddLinks( list<blk_ref> const & new_links ) = 0;

	/*!
	 * Used to remove <i>all</i> links from the owner block of this attribute to any other block.
	 *
	 * <b>Example:</b> 
	 * \code
	 * blk_ref my_block = ReadNifTree("test_in.nif");
	 * my_block->ClearLinks();
	 * \endcode
	 * 
	 * <b>In Python:</b>
	 * \code
	 * my_block = ReadNifTree("test_in.nif")
	 * my_block.ClearLinks();
	 * \endcode
	 * \sa IAttr:Set( blk_ref const & ), IAttr::HasLinks, IAttr::asLink, IAttr::asLinkList, IAttr::AddLink, IAttr::AddLinks, IAttr::RemoveLinks, IAttr::FindLink
	 */
	virtual void ClearLinks() = 0;

	/*!
	 * Used to remove all links between a particular block and this attributes owner block.  A block should never be linked multiple times to the same block, but this function will remove any accidental duplicates.
	 * \param block A reference to the block whose links are to be removed from this block.
	 *
	 * <b>Example:</b> 
	 * \code
	 * blk_ref my_block = ReadNifTree("test_in.nif");
	 * blk_ref block2 = CreateBlock("NiNode");
	 * my_block["Children"]->AddLink(block2);
	 * my_block["Children"]->RemoveLinks(block2);
	 * \endcode
	 * 
	 * <b>In Python:</b>
	 * \code
	 * my_block = ReadNifTree("test_in.nif")
	 * block2 = CreateBlock("NiNode")
	 * my_block.["Children"].AddLink(block2);
	 * my_block.["Children"].RemoveLinks(block2);
	 * \endcode
	 * \sa IAttr:Set( blk_ref const & ), IAttr::HasLinks, IAttr::asLink, IAttr::asLinkList, IAttr::AddLink, IAttr::AddLinks, IAttr::ClearLinks, IAttr::FindLink
	 */
	virtual void RemoveLinks( blk_ref const & block ) = 0;

	/*!
	 * Used to conveniently find the first link of a particular block type.  Especially convenient for property groups which should have only one property of each type.
	 * \param block_type The type of block to search for. Ex. NiMaterialProperty, NiTexturingProperty, NiShadeProperty, etc.
	 *
	 * <b>Example:</b> 
	 * \code
	 * blk_ref my_block = ReadNifTree("test_in.nif");
	 * blk_ref material = my_block->FindLink("NiMaterialProperty");
	 * \endcode
	 * 
	 * <b>In Python:</b>
	 * \code
	 * my_block = ReadNifTree("test_in.nif")
	 * material = my_block.FindLink("NiMaterialProperty")
	 * \endcode
	 * \sa IAttr:Set( blk_ref const & ), IAttr::HasLinks, IAttr::asLink, IAttr::asLinkList, IAttr::AddLink, IAttr::AddLinks, IAttr::ClearLinks, IAttr::RemoveLinks, IAttr::FindLink
	 */
	virtual blk_ref FindLink( string const & block_type ) const = 0;

	//Python Operator Overloads
	string __str__() {
		return asString();
	};
};

/*! 
 * An advanced interface for blocks that are nodes.
 * \sa IBlock::QueryInterface, QueryNode
 */
class INode {
public:
	INode() {}
	virtual ~INode() {}

	/*! 
	 * This is a conveniance function that allows you to retrieve the full 4x4 matrix transform of a node.  It accesses the "Rotation," "Translation," and "Scale" attributes and builds a complete 4x4 transformation matrix from them.
	 * \return A 4x4 transformation matrix built from the node's transform attributes.
	 */
	virtual Matrix44 GetLocalTransform() const = 0;

	/*! 
	 * This function will return a transform matrix that represents the location of this node in world space.  In other words, it concatenates all parent transforms up to the root of the scene to give the ultimate combined transform from the origin for this node.
	 * \return The 4x4 world transform matrix of this node.
	 */
	virtual Matrix44 GetWorldTransform() const = 0;

	/*!
	 * This function returns the bind position world matrix.  The bind position (also called the rest position) is the position of an object in a skin and bones system before any posing has been done.
	 * \return The 4x4 world bind position matrix of this node.
	 * \sa INode::GetLocalBindPos, INode::SetWorldBindPos
	 */
	virtual Matrix44 GetWorldBindPos() const = 0;

	/*!
	 * This function sets the bind position of this object relative to the origin.  The bind position (also called the rest position) is the position of an object in a skin and bones system before any posing has been done.  This function must be called on every object in a skin and bones system (the bones and the skinned shapes) in order for skinning information to be written to a Nif file.
	 * \param m The 4x4 world bind position matrix of this node
	 * \sa INode::GetLocalBindPos, INode::GetWorldBindPos
	 */
	virtual void SetWorldBindPos( Matrix44 const & m ) = 0;

	/*! This function returns the bind position world matrix of this node multiplied with the inverse of the bind position world matrix of its parent object if any.  Thus it returns the bind position of the object in local coordinates.  The bind position (also called the rest position) is the position of an object in a skin and bones system before any posing has been done.
	 * \return The 4x4 local bind position matrix of this node.
	 * \sa INode::SetWorldBindPos, INode::GetWorldBindPos
	 */
	virtual Matrix44 GetLocalBindPos() const = 0;
};

class IShapeData {
public:
	IShapeData() {}
	virtual ~IShapeData() {}
	//Counts
	virtual short GetVertexCount() const = 0;
	virtual short GetUVSetCount() const = 0;
	virtual void SetVertexCount(int n) = 0;
	virtual void SetUVSetCount(int n) = 0;
	//Getters
	virtual vector<Vector3> GetVertices() const = 0;
	virtual vector<Vector3> GetNormals() const = 0;
	virtual vector<Color4> GetColors() const = 0;
	virtual vector<TexCoord> GetUVSet( int index ) const = 0;
	//Setters
	virtual void SetVertices( const vector<Vector3> & in ) = 0;
	virtual void SetNormals( const vector<Vector3> & in ) = 0;
	virtual void SetColors( const vector<Color4> & in ) = 0;
	virtual void SetUVSet( int index, const vector<TexCoord> & in ) = 0;
};

class ITriShapeData {
public:
	ITriShapeData() {}
	virtual ~ITriShapeData () {}
	//Counts
	virtual short GetTriangleCount() const = 0;
	virtual void SetTriangleCount(int n) = 0;
	//Match Detection
	virtual void SetMatchDetectionMode(bool choice) = 0;
	virtual bool GetMatchDetectionMode() const = 0;
	//Getters
	virtual vector<Triangle> GetTriangles() const = 0;
	//Setters
	virtual void SetTriangles( const vector<Triangle> & in ) = 0;
};

class ITriStripsData {
public:
	ITriStripsData() {}
	virtual ~ITriStripsData () {}
	//Counts
	virtual short GetTriangleCount() const = 0;
	virtual short GetStripCount() const = 0;
	virtual void SetStripCount(int n) = 0;
	//Getter
	virtual vector<short> GetStrip( int index ) const = 0;
	virtual vector<Triangle> GetTriangles() const = 0;
	//Setter
	virtual void SetStrip( int index, const vector<short> & in ) = 0;
};

class ISkinData {
public:
	ISkinData() {}
	virtual ~ISkinData () {}
	virtual vector<blk_ref> GetBones() = 0; // Can't be const, since it changes the bone blk_ref reference 
	virtual map<int, float> GetWeights( blk_ref const & bone ) const = 0;
	virtual void AddBone( blk_ref const & bone, map<int, float> const & in ) = 0;
	virtual void RemoveBone( blk_ref const & bone ) = 0;
};

class IKeyframeData {
public:
	IKeyframeData() {}
	virtual ~IKeyframeData () {}
	//Rotate
	virtual KeyType GetRotateType() const = 0;
	virtual void SetRotateType( KeyType t ) = 0;
	virtual vector< Key<Quaternion> > GetRotateKeys() const = 0;
	virtual void SetRotateKeys( vector< Key<Quaternion> > const & keys ) = 0;
	//Translate
	virtual KeyType GetTranslateType() const = 0;
	virtual void SetTranslateType( KeyType t ) = 0;
	virtual vector< Key<Vector3> > GetTranslateKeys() const = 0;
	virtual void SetTranslateKeys( vector< Key<Vector3> > const & keys ) = 0;
	//Scale
	virtual KeyType GetScaleType() const = 0;
	virtual void SetScaleType( KeyType t ) = 0;
	virtual vector< Key<float> > GetScaleKeys() const = 0;
	virtual void SetScaleKeys( vector< Key<float> > const & keys ) = 0;
};

class ITextKeyExtraData {
public:
	ITextKeyExtraData() {}
	virtual ~ITextKeyExtraData () {}
	virtual vector< Key<string> > GetKeys() const = 0;
	virtual void SetKeys( vector< Key<string> > const & keys ) = 0;

};

class IColorData {
public:
	IColorData() {}
	virtual ~IColorData () {}
	virtual KeyType GetKeyType() const = 0;
	virtual void SetKeyType( KeyType t ) = 0;
	virtual vector< Key<Color4> > GetKeys() const = 0;
	virtual void SetKeys( vector< Key<Color4> > const & keys ) = 0;
};

class IFloatData {
public:
	IFloatData() {}
	virtual ~IFloatData () {}
	virtual KeyType GetKeyType() const = 0;
	virtual void SetKeyType( KeyType t ) = 0;
	virtual vector< Key<float> > GetKeys() const = 0;
	virtual void SetKeys( vector< Key<float> > const & keys ) = 0;
};

class IPosData {
public:
	IPosData() {}
	virtual ~IPosData () {}
	virtual KeyType GetKeyType() const = 0;
	virtual void SetKeyType( KeyType t ) = 0;
	virtual vector< Key<Vector3> > GetKeys() const = 0;
	virtual void SetKeys( vector< Key<Vector3> > const & keys ) = 0;
};


class IMorphData {
public:
	IMorphData() {}
	virtual ~IMorphData () {}
	virtual int GetVertexCount() const = 0;
	virtual void SetVertexCount( int n ) = 0;
	virtual int GetMorphCount() const = 0;
	virtual void SetMorphCount( int n ) = 0;
	virtual vector< Key<float> > GetMorphKeys( int n ) const = 0;
	virtual void SetMorphKeys( int n, vector< Key<float> > const & keys ) = 0;
	virtual vector<Vector3> GetMorphVerts( int n) const = 0;
	virtual void SetMorphVerts( int n, vector<Vector3> const & in ) = 0;
};

//struct ComplexVertex {
//	ComplexVertex() : has_color(false), has_normal(false), vertex_index(0), normal_index(0), color_index(0) {}
//	~ComplexVertex();
//	bool has_color, has_normal;
//	int vertex_index, color_index, normal_index;
//	vector<int> uv_indices;
//}
//
//struct ComplexFace {
//	vector<ComplexVertex> points;
//	int base_map_index;
//	int glow_map_index;
//};
//
//class ComplexShape {
//
//	void CombineTriShapes( list<blk_ref> & tri_shapes );
////	void SetVertices( vector<Vector3> & vertices );
////	void SetUVs( vector<TexCoord> & uvs );
////	void SetColors( vector<Color4> & colors );
////	void SetNormals( vector<Vector3> & normals );
////	void SetBones( vector<blk_ref> & bones );
////	void SetFaces( list< vector< ComplexVertex > > & faces );
////
////	vector<Vector3> GetVertices();
////	vector<TexCoord> GetUVs();
////	vector<Color4> GetColors();
////	vector<Vector3> GetNormals();
////	vector<blk_ref> GetBones();
////	list< vector< ComplexVertex > > GetFaces();
//
//private:
//	vector<Vector3> _vertices;
//	vector<Color4> _colors;
//	vector<Vector3> _normals;
//	list<ComplexFace> _faces;
//	map<string, blk_ref> _materials;
//	map<string, vector<TexCoord> > _uvs;
//	map<blk_ref, map<int, float> > _bones;
//};
//
//void ComplexShape::CombineTriShapes( list<blk_ref> & tri_shapes ) {
//	//Clear all internal datea
//	_vertices.clear();
//	_colors.clear();
//	_normals.clear();
//	_materials.clear();
//	_uvs.clear();
//	_faces.clear();
//	_bones.clear();
//
//	//Create a temporary spot to hold the triangle lists from each TriShape
//	vector< vector<Triangle> > ts_faces;
//
//	//Create lists to hold the lookup tables
//	vector<int> tri_lookup, nor_lookup, col_lookup;
//	map<string, vector<int> > mat_lookup, uv_lookup;
//	
//	//--Cycle through all the TriShapes, adding their data to the lists--//
//	list<blk_ref>::iterator it;
//
//	for (it = tri_shapes.begin(); it != tri_shapes.end(); ++it) {
//		ITriShapeData * data = QueryTriShapeData(*it);
//
//		//Vertices
//		vector<Vector3> ts_verts = data->GetVertices();
//		_vertices.insert(_vertices.end(), ts_verts.begin(), ts_verts.end();
//
//		//Normals
//		vector<Vector3> ts_norms = data->GetNormals();
//		_normals.insert(_normals.end(), ts_norms.begin(), ts_norms.end();
//		
//		//Colors
//		vector<Colors> ts_cols = data->GetColors();
//		_colors.insert(_colors.end(), ts_colors.begin(), ts_colors.end();
//
//		//Triangles
//		ts_faces[i] = data->GetTriangles();
//
//		//UV Coords
//		vector< vector<TexCoords> > uvs(data->GetUVSetCount());
//		for (int i = 0; i < data->GetUVSetCount(); ++i) {
//			uvs[i] = data->GetUVSet(i);
//		}
//
//		//Associate UV Coord Data with proper map name
//		blk_ref tx_prop = par["Properties"]->FindLink( "NiTexturingProperty");
//		if ( tx_prop.is_null() == false ) {
//			int uv_set = 0;
//			for (int i = 0; i < 7; ++i) {
//				string attr_name, map;
//				switch(i) {
//					case 0:	attr_name = "Base Texture";     map = "map1";   break;
//					case 1:	attr_name = "Dark Texture";     map = "dark";   break;
//					case 2:	attr_name = "Detail Texture";   map = "detail"; break;
//					case 3:	attr_name = "Gloss Texture";    map = "gloss";  break;
//					case 4:	attr_name = "Glow Texture";     map = "glow";   break;
//					case 5:	attr_name = "Bump Map Texture"; map = "bump";   break;
//					case 6:	attr_name = "Decal 0 Texture";  map = "decal0";
//				}
//
//				if ( tx_prop[attr_name]->asTexDesc().isUsed == true ) {
//					//How to merge all UVs?
//				}
//
//		}
//
//
//		//blk_ref material = (*it)->GetAttr("Propreties")->FindLink("NiMaterialProperty");
//		//blk_ref skin_inst = (*it)->GetAttr("Skin Instance")->asLink();
//		//blk_ref skin_data;
//		//vector<blk_ref> bones;
//		//map<int, float> weights;
//		//if ( skin_inst.is_null() == false ) {
//		//	skin_block = skin_inst->GetAttr("Data");
//		//	if (skin_block.is_null() == false ) {
//		//		ISkinData * skin_data = QuerySkinData(skin_block);
//		//		weights = skin_data->GetWeights();
//		//		bones = skin_data->GetBones();
//		//	}
//		//}
//
//	}
//}

//--Attribute Reference--//
/*!
 * Stores a reference to an attribute in a NIF block and offers functions to help use it.
 * \sa IAttr
 */
class attr_ref {
public:
	/*! Default constructor.  Initializes the attribute reference in null state. */
	attr_ref() : _attr(NULL) {}

	attr_ref( IAttr * ptr ) : _attr(ptr) {}

	/*! This operator allows you to use an attribute reference as if it were a pointer to an IAttr interface.  You can call any IAttr function by using it.
	 * \return An IAttr pointer which can be used to call functions.
	 * \sa IAttr
	 */
	IAttr * operator->() const {
		if ( _attr == NULL ) {
			//pointer has not been fixed, throw exception
			throw runtime_error("Attempted to dereference a null Attribute reference.");
		} else {
			return _attr;
		}
	}

	/*!
	 * Used to determine whether an attribute reference is null, i.e. it points to nothing and thus no functions can be called on it.
	 * \return True if the attribute is null and false otherwise.
	 *
	 * <b>Example:</b> 
	 * \code
	 * blk_ref my_block = ReadNifTree("test_in.nif");
	 * attr_ref my_attr = my_block["NiNode"];
	 * if ( my_attr.is_null() == true )
	 *      cout << "Null Attribute" << endl;
	 * \endcode
	 * 
	 * <b>In Python:</b>
	 * \code
	 * my_block = ReadNifTree("test_in.nif")
	 * my_attr = my_block["NiNode"]
	 * if attr.is_null() == True:
	 *      print "Null Attribute"
	 * \endcode
	 */
	bool is_null() const {
		if (_attr == NULL )
			return true;
		else
			return false;
	}
	IAttr * ptr() const {
		return _attr;
	}

	//--Comparison operators--//

	/*! This operator allows you to compare two attribute references to see if they point to the same attribute.  It does not compare the values held by the attributes.
	 * \param rh The right hand operand to the == operator.
	 * \return True if both references point to the same attribute.  False otherwise.
	 */
	bool operator==(attr_ref & rh) const {
		if (_attr == rh._attr)
			return true;
		else
			return false;
	}

	/*! This operator alows you to compare two attribute references to see if they <i>do not</i> point to the same attribute.  It does not compare the values held by the attributes.
	 * \param rh The right hand operand to the != operator.
	 * \return True if both references <i>do not</i> point to the same attribute.  False otherwise.
	 */
	bool operator!=(attr_ref & rh) const {
		if (_attr != rh._attr)
			return true;
		else
			return false;
	}

	/*! This operator is provided to allow associative STL containers to use attribute references as keys.  The comparison is based on the value of the IAtter pointer held by each reference.
	 * \param rh The right hand operand to the < operator.
	 * \return True if the pointer value of the left hand reference holds a pointer value that is less than that held by the right hand operand.  False otherwise.
	 */
	bool operator<(attr_ref & rh) const {
		if (_attr < rh._attr)
			return true;
		else
			return false;
	}

	//--Assignment operators--//

	/*! The assignment operators are shorthand for using the IAttr::Set function.  They allow you to set the value contained by the attribute pointed to by this reference by using the = operator instead of calling their Set function through the -> operator.
	 * \param n The new value to set the IAttr attribute to using its Set function.
	 * \return a reference to this attr_ref object.
	 */
	attr_ref & operator=(int n) {
		_attr->Set(n);
		return *this;
	}

	/*! The assignment operators are shorthand for using the IAttr::Set function.  They allow you to set the value contained by the attribute pointed to by this reference by using the = operator instead of calling their Set function through the -> operator.
	 * \param n The new value to set the IAttr attribute to using its Set function.
	 * \return a reference to this attr_ref object.
	 */
	attr_ref & operator=(float n) {
		_attr->Set(n);
		return *this;
	}

	/*! The assignment operators are shorthand for using the IAttr::Set function.  They allow you to set the value contained by the attribute pointed to by this reference by using the = operator instead of calling their Set function through the -> operator.
	 * \param n The new value to set the IAttr attribute to using its Set function.
	 * \return a reference to this attr_ref object.
	 */
	attr_ref & operator=(Float3 const & n) {
		_attr->Set(n);
		return *this;
	}

	/*! The assignment operators are shorthand for using the IAttr::Set function.  They allow you to set the value contained by the attribute pointed to by this reference by using the = operator instead of calling their Set function through the -> operator.
	 * \param n The new value to set the IAttr attribute to using its Set function.
	 * \return a reference to this attr_ref object.
	 */
	attr_ref & operator=(string const & n) {
		_attr->Set(n);
		return *this;
	}

	/*! The assignment operators are shorthand for using the IAttr::Set function.  They allow you to set the value contained by the attribute pointed to by this reference by using the = operator instead of calling their Set function through the -> operator.
	 * \param n The new value to set the IAttr attribute to using its Set function.
	 * \return a reference to this attr_ref object.
	 */
	attr_ref & operator=(const char * n) {
		_attr->Set( string(n) );
		return *this;
	}

	/*! The assignment operators are shorthand for using the IAttr::Set function.  They allow you to set the value contained by the attribute pointed to by this reference by using the = operator instead of calling their Set function through the -> operator.
	 * \param n The new value to set the IAttr attribute to using its Set function.
	 * \return a reference to this attr_ref object.
	 */
	attr_ref & operator=(Matrix33 const & n) {
		_attr->Set(n);
		return *this;
	}

	/*! The assignment operators are shorthand for using the IAttr::Set function.  They allow you to set the value contained by the attribute pointed to by this reference by using the = operator instead of calling their Set function through the -> operator.
	 * \param n The new value to set the IAttr attribute to using its Set function.
	 * \return a reference to this attr_ref object.
	 */
	attr_ref & operator=(blk_ref const & n) {
		_attr->Set(n);
		return *this;
	}

	/*! The assignment operators are shorthand for using the IAttr::Set function.  They allow you to set the value contained by the attribute pointed to by this reference by using the = operator instead of calling their Set function through the -> operator.
	 * \param n The new value to set the IAttr attribute to using its Set function.
	 * \return a reference to this attr_ref object.
	 */
	attr_ref & operator=(TexSource const & n) {
		_attr->Set(n);
		return *this;
	}

	/*! The assignment operators are shorthand for using the IAttr::Set function.  They allow you to set the value contained by the attribute pointed to by this reference by using the = operator instead of calling their Set function through the -> operator.
	 * \param n The new value to set the IAttr attribute to using its Set function.
	 * \return a reference to this attr_ref object.
	 */
	attr_ref & operator=(BoundingBox const & n) {
		_attr->Set(n);
		return *this;
	}

	/*! The assignment operators are shorthand for using the IAttr::Set function.  They allow you to set the value contained by the attribute pointed to by this reference by using the = operator instead of calling their Set function through the -> operator.
	 * \param n The new value to set the IAttr attribute to using its Set function.
	 * \return a reference to this attr_ref object.
	 */
	attr_ref & operator=(ConditionalInt const & n) {
		_attr->Set(n);
		return *this;
	}

	/*! The assignment operators are shorthand for using the IAttr::Set function.  They allow you to set the value contained by the attribute pointed to by this reference by using the = operator instead of calling their Set function through the -> operator.
	 * \param n The new value to set the IAttr attribute to using its Set function.
	 * \return a reference to this attr_ref object.
	 */
	attr_ref & operator=(TexDesc const & n) {
		_attr->Set(n);
		return *this;
	}

	//--Conversion fuctions--//

	/*! The type operators are shorthand for using the IAttr::AsType functions.  They allow you to retrieve the value contained by the attribute pointed to by this reference by using the = operator instead of calling their asType function through the -> operator.
	 * \return the value stored in the attribute stored in this reference by calling its asType function.
	 */
	operator int() const { return _attr->asInt(); }

	/*! The type operators are shorthand for using the IAttr::AsType functions.  They allow you to retrieve the value contained by the attribute pointed to by this reference by using the = operator instead of calling their asType function through the -> operator.
	 * \return the value stored in the attribute stored in this reference by calling its asType function.
	 */
	operator float() const { return _attr->asFloat(); }

	/*! The type operators are shorthand for using the IAttr::AsType functions.  They allow you to retrieve the value contained by the attribute pointed to by this reference by using the = operator instead of calling their asType function through the -> operator.
	 * \return the value stored in the attribute stored in this reference by calling its asType function.
	 */
	operator Float3() const { return _attr->asFloat3(); }

	/*! The type operators are shorthand for using the IAttr::AsType functions.  They allow you to retrieve the value contained by the attribute pointed to by this reference by using the = operator instead of calling their asType function through the -> operator.
	 * \return the value stored in the attribute stored in this reference by calling its asType function.
	 */
	operator string() const { return _attr->asString(); }
	
	/*! The type operators are shorthand for using the IAttr::AsType functions.  They allow you to retrieve the value contained by the attribute pointed to by this reference by using the = operator instead of calling their asType function through the -> operator.
	 * \return the value stored in the attribute stored in this reference by calling its asType function.
	 */
	operator Matrix33() const { return _attr->asMatrix33(); }

	/*! The type operators are shorthand for using the IAttr::AsType functions.  They allow you to retrieve the value contained by the attribute pointed to by this reference by using the = operator instead of calling their asType function through the -> operator.
	 * \return the value stored in the attribute stored in this reference by calling its asType function.
	 */
	operator blk_ref() const;

	/*! The type operators are shorthand for using the IAttr::AsType functions.  They allow you to retrieve the value contained by the attribute pointed to by this reference by using the = operator instead of calling their asType function through the -> operator.
	 * \return the value stored in the attribute stored in this reference by calling its asType function.
	 */
	operator TexSource() const;

	/*! The type operators are shorthand for using the IAttr::AsType functions.  They allow you to retrieve the value contained by the attribute pointed to by this reference by using the = operator instead of calling their asType function through the -> operator.
	 * \return the value stored in the attribute stored in this reference by calling its asType function.
	 */
	operator BoundingBox() const;

	/*! The type operators are shorthand for using the IAttr::AsType functions.  They allow you to retrieve the value contained by the attribute pointed to by this reference by using the = operator instead of calling their asType function through the -> operator.
	 * \return the value stored in the attribute stored in this reference by calling its asType function.
	 */
	operator ConditionalInt() const;

	/*! The type operators are shorthand for using the IAttr::AsType functions.  They allow you to retrieve the value contained by the attribute pointed to by this reference by using the = operator instead of calling their asType function through the -> operator.
	 * \return the value stored in the attribute stored in this reference by calling its asType function.
	 */
	operator TexDesc() const;

	/*! The type operators are shorthand for using the IAttr::AsType functions.  They allow you to retrieve the value contained by the attribute pointed to by this reference by using the = operator instead of calling their asType function through the -> operator.
	 * \return the value stored in the attribute stored in this reference by calling its asType function.
	 */
	operator list<blk_ref>() const { return _attr->asLinkList(); }

	//friend ostream & operator<<(ostream & lh, const attr_ref & rh) {
	//	if ( rh._attr == NULL )
	//		lh << "Null Attribute";
	//	else
	//		lh << rh->GetName() << ":  " << rh->asString();
	//	}
	//	return lh;
	//}

private:
	IAttr * _attr;
};

//--Block Reference--//

/*! An automatic pointer that stores a reference to a particular NIF block.  It uses reference counting to keep track of when a block should be destroyed. 
 * \sa IBlock
 */
class blk_ref {
public:
	/*! Default constructor.  Initializes reference to null state. */
	blk_ref() : _index(-1), _block(NULL) {}
	blk_ref( int index ) : _index(index), _block(NULL) {}
	
	//--Copy Constructor--//

	/*! The copy constructor allows you to copy one block reference to another.  It does not actually makea copy of the data, it only creates a new pointer and increments the reference count of the existing data.
	 * \param rh The right hand operand to the = operator
	 */
	blk_ref(const blk_ref & rh ) {
		_block = rh._block;
		_index = rh._index;

		if ( _block != NULL ) {
			//cout << "Copying from another reference - ";
			//Increment reference count
			_block->AddRef();
		}
	}

	//--Asignment Operator--//

	/*! The asignment operator allows you to assign one block reference to another.  It does not actually make a copy of the data, it only changes the value of one pointer, causing the reference count on the block it originally pointed to to go down, and the reference count of the new block it points to to go up.
	 * \param rh The right hand operand to the = operator
	 * \return a reference to this object.
	 */
	blk_ref & operator=(const blk_ref & rh ) {
		//Make sure it's not already equal
		if (_block == rh._block && get_index() == rh.get_index() ) {
			// already equal
			return *this;
		}

		// the incoming reference is different
		if ( _block != NULL ) {
			//Decrement reference count
			//cout << "This reference was overwritten by another - ";
			_block->SubtractRef();
		}
		_block = rh._block;
		_index = rh._index;
		if (_block != NULL ) {
			//rhcrement reference count
			//cout << "Copied from another reference using = - ";
			_block->AddRef();
		}

		return *this;
	}

	//--Equality Operator--//

	/*! The equality operator allows you to determine whether two references point to the same block.  It does not compare data within the blocks.
	 * \param rh The right hand operand to the == opearator.
	 * \return True if both references point to the same block.  False otherwise.
	 */
	bool operator==(const blk_ref & rh) const {
		if ( _block == rh._block && _index == rh._index )
			return true;
		else
			return false;
	}

	// Inequality Operator

	/*! The inequality operator allows you to determine whether two references <i>do not</i> point to the same block.  It does not compare data within the blocks.
	 * \param rh The right hand operand to the != opearator.
	 * \return True if both references <i>do not</i> point to the same block.  False otherwise.
	 */
	bool operator!=(const blk_ref & rh) const {
		return !(*this == rh);
	}

	// Less-Than Operator

	/*! The less-than opearator is provided to allow associative STL containers to use block references as keys.  The comparison is based on the value of the IBlock pointer held by each reference.
	 * \param rh The right hand operand to the < operator.
	 * \return True if the pointer value of the left hand reference holds a pointer value that is less than that held by the right hand operand.  False otherwise.
	 */
	bool operator<( const blk_ref & rh ) const {
		return ( _block < rh._block );
	}
	
	/*!
	 * Used to determine whether a block reference is null, i.e. it points to nothing and thus no functions can be called on it.
	 * \return True if the attribute is null and false otherwise.
	 */
	bool is_null() const {
		if (_block == NULL && _index == -1)
			return true;
		else
			return false;
	}
	blk_ref( IBlock * block ) : _index(-1), _block(block) {

		//Increment reference count
		//cout << "Creating from block - ";
		_block->AddRef();
	}

	/*! Default destructor.  Automatically decrements reference counts, causing the IBlock to destroy itself if the count drops to zero. */
	~blk_ref() {
		//If pointer has been fixed
		if ( _block != NULL ) {
			//Decrement reference count
			//cout << "Reference deleted - ";
			_block->SubtractRef();
		}
	}

	void set_block( IBlock * block ) {
		//Use assignment operator code
		*this = blk_ref( block );
	}

	/*! The dereference operator allows you to use a block reference as if it were a pointer to an IBlock interface.  You can call any IBlock function by using it.
	 * \return An IBlock pointer which can be used to call functions.
	 * \sa IBlock
	 */
	IBlock * operator->() const {
		if ( _block == NULL ) {
			//pointer has not been fixed, throw exception
			cout << *this << endl;
			throw runtime_error("Attempted to dereference a block reference that has not been fixed.");
		} else {
			return _block;
		}
	}

	/*! The bracket operator allows you to use a block reference as if it were an associative array of attribute values.  It is shorthand for calling the IBlock::GetAttr function with the name of the attribute you want to retrieve.
	 * \param index The name of the attribute you want to retrieve with the GetAttr function.  Ex. "Scale," "Skeleton Root," "Controller," etc.
	 * \return An attribute reference pointing to the attribute that was requested.  If the attribute is not present, a null reference is returned.
	 * \sa IBlock::GetAttr
	 */
	attr_ref operator[] ( string const & index ) const {
		return _block->GetAttr(index);
	}

	int get_index() const {

		if (_block != NULL)
			return _block->GetBlockNum();
		else
			return _index; 
	}

	IBlock * get_block() const {
		return _block;
	}

	void set_index( int index ) { 
		//Use assigntment operator code
		*this = blk_ref( index );
	}

	bool is_fixed() const {
		if ( _block == NULL )
			return false;
		else
			return true;
	}

	/*! The << stream operator allows you to use a block reference in standard C++ string stream operations.  It causes a short description of the block that is pointed to by this reference to be writen to the given ostream.  Ex:  NiNode(Block 14) [Pelvis]
	 * \param lh The left hand operand to the << operator, the ostream to write the short description to.
	 * \param rh The right hand operand to the << operator, the blk_ref whos short description is to be written.
	 * \return The same ostream that was given in lh so that stream operations can be concatenated.
	 */
	friend ostream & operator<<(ostream & lh, const blk_ref & rh) {
		if (rh._block != NULL) {
			lh << rh._block->GetBlockType() << "(Block " << rh._block->GetBlockNum() << ")";
			
			attr_ref attr = rh._block->GetAttr("Name");
			if ( attr.is_null() == false ) {
				lh << " [" << attr->asString() << "]";
			}
		} else {
			if (rh._index == -1)
				lh << "Null (-1)";
			else
				lh << "Block " << rh._index;
		}
		return lh;
	}

	//Python Operator Overloads
	attr_ref __getitem__(string index) {
		return _block->GetAttr(index);
	}
	void __setitem__(string index, int value) {
		attr_ref attr = _block->GetAttr(index);
		if ( attr.is_null() == true )
			throw std::out_of_range("Tried to set an attribute via [] that does not exist in this block.");
		attr->Set(value);
	}
	void __setitem__(string index, float value) {
		attr_ref attr = _block->GetAttr(index);
		if ( attr.is_null() == true )
			throw std::out_of_range("Tried to set an attribute via [] that does not exist in this block.");
		attr->Set(value);
	}
	void __setitem__(string index, Float3 value) {
		attr_ref attr = _block->GetAttr(index);
		if ( attr.is_null() == true )
			throw std::out_of_range("Tried to set an attribute via [] that does not exist in this block.");
		attr->Set(value);
	}
	void __setitem__(string index, string value) {
		attr_ref attr = _block->GetAttr(index);
		if ( attr.is_null() == true )
			throw std::out_of_range("Tried to set an attribute via [] that does not exist in this block.");
		attr->Set(value);
	}
	void __setitem__(string index, Matrix33 value) {
		attr_ref attr = _block->GetAttr(index);
		if ( attr.is_null() == true )
			throw std::out_of_range("Tried to set an attribute via [] that does not exist in this block.");
		attr->Set(value);
	}
	void __setitem__(string index, blk_ref value) {
		attr_ref attr = _block->GetAttr(index);
		if ( attr.is_null() == true )
			throw std::out_of_range("Tried to set an attribute via [] that does not exist in this block.");
		attr->Set(value);
	}
	void __setitem__(string index, TexSource const & value) {
		attr_ref attr = _block->GetAttr(index);
		if ( attr.is_null() == true )
			throw std::out_of_range("Tried to set an attribute via [] that does not exist in this block.");
		attr->Set(value);
	}
	void __setitem__(string index, BoundingBox const & value) {
		attr_ref attr = _block->GetAttr(index);
		if ( attr.is_null() == true )
			throw std::out_of_range("Tried to set an attribute via [] that does not exist in this block.");
		attr->Set(value);
	}
	void __setitem__(string index, ConditionalInt const & value) {
		attr_ref attr = _block->GetAttr(index);
		if ( attr.is_null() == true )
			throw std::out_of_range("Tried to set an attribute via [] that does not exist in this block.");
		attr->Set(value);
	}
	void __setitem__(string index, TexDesc const & value) {
		attr_ref attr = _block->GetAttr(index);
		if ( attr.is_null() == true )
			throw std::out_of_range("Tried to set an attribute via [] that does not exist in this block.");
		attr->Set(value);
	}
protected:
	int _index;
	IBlock * _block;
};

#endif
